#! /usr/bin/python
#
# testports.py
# From testports.cfg obtain a set of links from [links] section and run the tests
# specified in [tests] on them. 
# The tool can be run on any machine that has python(and pexpect) and has network
# access to the switches mentioned below.
#
# Sample testports.cfg
#    [links]
#    act-7712-05,swp9s0 = act-7712-05,swp10s0
#    act-7712-05,swp13 = act-7712-05,swp14
#    10.42.0.86,swp1 = 10.42.0.86,swp2
#    10.42.0.86,swp52 = 10.42.0.86,swp51
#
#    [tests]
#    PortLinkUp = ALL
#    PortLinkDown = ALL
#    SwitchSwitchdRestart = ALL
#    SwitchPortwdRestart = ALL
#    PortLLDPCheck = ALL
#    PortLinkFlap = ALL
#    PortEEPROMCheck = ALL
#
# Examples:
#  - List of tests available with description
#  ./testports.py -t or ./testports.py --tests
#
# - List of links to be tested
# ./testports.py -l or ./testports.py --links
#
# - Run the tests
# ./testports.py -r or ./testports.py --runtests
#
import argparse
import ConfigParser
import os
import sys
import time
import pexpect
import re
from pexpect import pxssh
from datetime import datetime

# Base Switch Infra Class
class Switch(object):
    def __init__(self, sw1, logf):
        ''' Base Switch Class. Can also be used for server.

        name       - Name of th switch
        ports      - dict() of test port on the switch 
        session    - session to send cmds to switch
        no_license - Check if switch has no license. Returns true/false.
        mgmtIP     - Mgmt IP on the switch. On error, return None
        chip       - switching asic on the switch. On error, return None
        platform   - platform name as per platform-detect. On error, return None
        release    - software release as /etc/lsb-release
        
        add_port(p)                        - Add port to ports dict
        daemon_start(d)/daemon_stop(d)
        daemon_restart(d)/deamon_status(d) - control daemon on switch
        log(tst, msg)                      - Write msgs to log file
        log_err(tst, msg)                  - Write error msgs to log file

        '''
        self.name = sw1
        self.logf = logf
        self.ports = dict()

        if self.login('root', 'cn321', sudo=False):
            if self.login('root', 'cn321', sudo=False, prompt='#'):
                if self.login('cumulus', 'CumulusLinux!'):
                    if self.login('cumulus', 'cn321'):
                        print 'Exiting due to login failure at %s' %self.name
                        sys.exit(1)

    def login(self, user, passwd, prompt='$', sudo=True):
        err = False
        if pexpect.__version__.startswith('4'):            
            self.session = pxssh.pxssh(timeout=3,
                        options={"StrictHostKeyChecking": "no",
                        "UserKnownHostsFile": "/dev/null"})
        else:
            self.session = pxssh.pxssh(timeout=3)
        try:
            self.session.login(self.name, user, passwd,
                                login_timeout=3, original_prompt=r"[%s]" %prompt)
        except:
            return True
                
        if sudo:
            self.session.sendline('sudo -i')
            self.session.expect(['\[sudo\] password for cumulus\:', '#'])
            self.session.sendline('CumulusLinux!')
            self.session.prompt()
            self.session.set_unique_prompt()        
        return err
            
                        
    def __send_cmd(self, cmd, timeout=None):
        s = self.session
        s.sendline(cmd)
        if timeout:
            s.prompt(timeout=timeout)
        else:
            s.prompt()
        # Useful for debugging. Will show all the cmds sent to the switch and the output.
        # print s.before
        return s.before

    def __get_line(self, cmd, substr):
        before = self.__send_cmd(cmd)
        l = [line for line in before.split('\n') if substr in line]
        return l

    def add_port(self, port):
        self.ports[port.name] = port

    def daemon_status(self, daemon):
        l = self.__get_line('systemctl status %s | grep Active' %daemon,
                            "Active:")
        if l:
            return l[0].split()[1]
        return None

    def daemon_stop(self, daemon):
        self.__send_cmd('systemctl stop %s' %daemon)

    def daemon_start(self, daemon):
        self.__send_cmd('systemctl start %s' %daemon)

    def daemon_restart(self, daemon):
        self.__send_cmd('systemctl reset-failed')
        # daemons like switchd take 3-4 minutes to come up.
        self.__send_cmd('systemctl restart %s' %daemon, 240)

    @property
    def no_license(self):
        l = self.__get_line('cl-license', 'No license installed!')
        if l:
            return True
        return False

    @property
    def mgmtIP(self):
        l = self.__get_line('ifconfig eth0', 'inet addr:')
        if l:
            return l[0].split()[1].split(':')[1]
        return None

    @property
    def chip(self):
        l = self.__get_line('/usr/lib/cumulus/bcmcmd show unit', 'Unit 0')
        if l:
            return l[0].split()[3]
        return None

    @property
    def release(self):
        l = self.__get_line('cat /etc/lsb-release', 'DISTRIB_RELEASE')
        if l:
            return l[0].split('=')[1].strip()
        return None

    @property
    def platform(self):
        l = self.__get_line('platform-detect', ',')
        if l:
            return l[0].split('\n')[0].strip(' \t\n\r')
        return None

    def log(self, tst_name, msg):
        l_msg = '%s %s: %s %s\n' %(datetime.utcnow(), tst_name,
                                   self.name, msg)
        self.logf.write(l_msg)

    def log_err(self, tst_name, msg):
        ''' log error messagses
        '''
        l_msg = '%s %s %s: ERROR %s\n' %(datetime.utcnow(), tst_name,
                                         self.name, msg)
        self.logf.write(l_msg)

class Port(object):
    ''' Base Port Class

    name - name of the port, e.g. swp1
    switch - class of switch the port belongs to
    peer_port - class of peer port
    peer_switch - class of peer switch
    identifier - Module id
    trans_type - Module Transreceiver type
    vendor_name, vendor_pn, vendor_sn - Module vendor info
    speed - port speed from ethtool
    link - link state ('up' or 'down') from ethtool
    lldp_peer_port - Peer port obtained via lldpctl
    lldp_peer_MgmtIP - Peer MgmtIP obtained via lldpctl

    lldpcli_update - send lldp frames to peer
    log(tst, msg)  - Write msgs to log file
    log_err(tst, msg) - Write error msgs to log file
    
    '''
    def __init__(self, name, switch, logf):
        self.name = name
        self.switch = switch
        self.logf = logf
        self.peer_port = None
        self.peer_switch = None 

        self.identifier, self.trans_type, self.vendor_name, self.vendor_pn, \
        self.vendor_sn, self.length = self.__module_info()

    def add_peer(self, switch, port):
        self.peer_switch = switch
        self.peer_port = port

    def log(self, tst_name, msg):
        l_msg = '%s %s: %s %s\n' %(datetime.utcnow(), tst_name,
                                   self.switch.name +  ',' + self.name, msg)
        self.logf.write(l_msg)

    def log_err(self, tst_name, msg):
        l_msg = '%s %s %s: ERROR %s\n' %(datetime.utcnow(), tst_name,
                                         self.switch.name +  ',' + self.name, msg)
        self.logf.write(l_msg)        

    def __send_cmd(self, cmd, timeout=None):
        s = self.switch.session
        s.sendline(cmd)
        if timeout:
            s.prompt(timeout=timeout)
        else:
            s.prompt()
        # useful for debugging. Will show all the cmds sent to the switch and the output.            
        # print s.before
        return s.before
        
    def __get_line(self, cmd, substr):
        before = self.__send_cmd(cmd)
        l = [line for line in before.split('\n') if substr in line]
        return l

    @property
    def cage(self):
        # Port with EEPROMs have entries in sfptab
        l = self.__get_line('cat /var/lib/cumulus/sfptab', self.name)
        if not l:
            return 'RJ45'
        return l[0].split()[1].strip().upper()

    def __module_info(self):
        ident = None
        trans_type = None
        vendor_name = None
        vendor_pn = None
        vendor_sn = None
        length = None

        if self.cage == 'RJ45':            
            return None, None, None, None, None, None

        before = self.__send_cmd('ethtool -m %s' %self.name)
        l = [line for line in before.split('\n') if "Identifier" in line]
        if l:
            ident = l[0].split(':')[1].split()[1].strip(' \t\n\r()')
        l = [line for line in before.split('\n') if "Transceiver type" in line]
        if l:
          trans_type = l[0].split(':')[2].strip(' \t\n\r')
        l = [line for line in before.split('\n') if "Vendor name" in line]
        if l:
          vendor_name = l[0].split(':')[1].strip(' \t\n\r')
        l = [line for line in before.split('\n') if "Vendor PN" in line]
        if len(l):
          vendor_pn = l[0].split(':')[1].strip(' \t\n\r')
        l = [line for line in before.split('\n') if "Vendor SN" in line]
        if l:
          vendor_sn = l[0].split(':')[1].strip(' \t\n\r')
        l = [line for line in before.split('\n') if "Length (Copper or Active cable)" in line]
        if l:
          length = l[0].split(':')[1].strip(' \t\n\r')
                  
        return ident, trans_type, vendor_name, vendor_pn, vendor_sn, length

    def _get_link(self):
        l = self.__get_line('ethtool %s' %self.name, 'Link detected:')
        if not l:
            return None
        state = l[0].split()[2]
        if state == 'yes':
            return 'up'
        elif state == 'no':
            return 'down'
        else:
            return state

    def _set_link(self, state):
        if state not in ('up', 'down'):
            self.log_err('', '_set_link value should be up or down not %s' %state)
            return
        self.__send_cmd('ip link set %s %s' %(self.name, state))

    link = property(_get_link, _set_link)

    def _get_speed(self):
        l = self.__get_line('ethtool %s' %self.name, "Speed:")
        if l:
            speed = l[0].split()[1].strip()
            x = re.search(r'\d+', speed)
            if x is None:
                return speed
            return int(x.group())
        return None

    def _set_speed(self, speed):
        valid_speeds = [1000, 10000, 25000, 40000, 50000, 100000]
        if speed not in valid_speeds:
            self.log_err('', 'speed %s not in %s' %(speed, valid_speeds))
            return

        cmd = 'ethtool -s %s speed %s duplex full' %(self.name, speed)
        before = self.__send_cmd(cmd)
        l = [line for line in before.split('\n') if "not" in line]
        if l:
            self.log_err('', 'cmd (%s) failed with following msg: %s' %(cmd, before))

    speed = property(_get_speed, _set_speed)

    def _get_autoneg(self):
        l = self.__get_line('ethtool %s' %self.name, "Auto-negotiation:")
        if l:
            return l[0].split()[1].strip()
        return None

    def _set_autoneg(self, an):
        valid_ans = ['on', 'off']
        if an not in valid_ans:
            self.log_err('', 'autoneg %s not in %s' %(an, valid_ans))
            return

        cmd = 'ethtool -s %s autoneg %s' %(self.name, an)
        before = self.__send_cmd(cmd)
        l = [line for line in before.split('\n') if "not" in line]
        if l:
            self.log_err('', 'cmd (%s) failed with following msg: %s' %(cmd, before))

    autoneg = property(_get_autoneg, _set_autoneg)
            
    def lldpcli_update(self):
        self.__send_cmd('lldpcli update')
        
    @property
    def lldp_peer_port(self):
        l = self.__get_line('lldpctl %s' %self.name, 'PortDescr:')
        if l:
            return l[0].split()[1]
        return None

    @property
    def lldp_peer_MgmtIP(self):
        l = self.__get_line('lldpctl %s' %self.name, 'MgmtIP:')
        if l:
            return l[0].split()[1]
        return None    

    @property
    def max_speed(self):
        cage = self.cage
        identifier = self.identifier
        trans_type = self.trans_type
        if cage == 'QSFP28' and identifier == 'QSFP28':
            return 100000
        elif cage == 'QSFP28' and identifier == 'QSFP+':
            return 40000        
        elif cage == 'QSFP+':
            return 40000
        else: # Lets add the SFP+ and RJ45 port cases later ...
            return None
            
class BaseTest(object):

    def __init__(self, switch_dict, port_dict, params, logf):
        ''' Base Test Class

            switch_dict - Dictionary of all switch objects
            port_dict - Dictionary of all port objects
            test_ports - List of port objects to test
            test_switches - List of all  switch objects to test

            port_log(p, msg) - For port specific messages
            port_log_err(p, msg) - For port specific error messages
            switch_log(p, msg) - For switch specific messages
            switch_log_err(p, msg) - For switch specific error messages

            Each test will have 3 stages:
            pre() - Preprocessing to do before the test. (e.g. link UP for lldpctl test)
            run() - run the test
            post() - Clean up after the test.

            Test is considered failed if anyone of these stages returns 'True'. We do not continue
            if any one of the stages fail.
                        
        '''
        self.switch_dict = switch_dict
        self.port_dict = port_dict
        self.logf = logf
        self.test_ports = []
        self.test_switches = []
        
        for name, port in port_dict.iteritems():
            self.test_ports.append(port)

        for name, switch in switch_dict.iteritems():
            self.test_switches.append(switch)

    def port_log(self, port, msg):
        port.log(self.__class__.__name__, msg)

    def port_log_err(self, port, msg):
        port.log_err(self.__class__.__name__, msg)
        
    def switch_log(self, switch, msg):
        switch.log(self.__class__.__name__, msg)

    def switch_log_err(self, switch, msg):
        switch.log_err(self.__class__.__name__, msg)
                
    def log(self, msg):
        self.logf.write('%s %s: %s\n' %(datetime.utcnow(),
                                        self.__class__.__name__, msg))

    def log_err(self, msg):
        self.logf.write('%s %s: ERROR %s\n' %(datetime.utcnow(),
                                        self.__class__.__name__, msg))
                                            
    def pre(self):
        return False

    def run(self):
        return False

    def post(self):
        return False

class BasePortTest(BaseTest):
    ''' Base Port Test Class

        ports_status(st) - Checks if all ports have the link state same as 'state'
        ports_set(st)    -  Set the link state of all ports as 'state' and check status.
    '''
    def ports_status(self, state):
        if state not in ('up, down'):
            self.log('Incorrect link state specified: %s' %state)
            return True
        self.log('Checking if all ports are %s' %state)
        err = False
        for port in self.test_ports:
            if (port.link != state):
                self.port_log_err(port, 'Link is not %s' %state)
                err = True
        return err

    def ports_set(self, state):
        if state not in ('up, down'):
            self.log('Incorrect link state specified: %s' %state)
            return True        
        self.log('Bringing all ports %s' %state)
        for port in self.test_ports:
            port.link = state
        # Some ports specially the 10GT ports take time to come UP.
        self.log('Sleeping for 10 secs before checking if links are UP')
        time.sleep(10)
        self.log('Checking if links are %s ...' %state)
        return self.ports_status(state)

# Base Switch Test Class
class BaseSwitchTest(BasePortTest):
    ''' Base Switch Test Class

        switches_daemon_restart(d) - Restart daemon on all switches.
    '''

    def switches_daemon_restart(self, daemon):
        err = False
        self.log('Restarting %s on all switches' %daemon)
        for switch in self.test_switches:
            switch.daemon_restart(daemon)
        for switch in self.test_switches:
            status = switch.daemon_status(daemon)
            if  status != 'active':
                self.switch_log_err(switch, '%s status is %s, not active'
                                    %(daemon, status))
                err = True
        return err

# Let the tests begin
class PortLinkUp(BasePortTest):
    description = 'Test Port Link Down and UP'
    
    def pre(self):
        return self.ports_set('down')
                
    def run(self):
        return self.ports_set('up')

# Let the tests begin
class PortLinkFlap(BasePortTest):
    description = 'Flap link 20 times and check link is UP every time.'
    loop_cnt = 20
    
    def run(self):
        fail = False
        for idx in range(self.loop_cnt):
            if self.ports_set('down'):
                self.log_err('Port Down failed on iteration: %d\n' %idx)
                fail = True                
            if self.ports_set('up'):
                self.log_err('Port Up failed on iteration: %d\n' %idx)
                fail = True                
        return fail

class PortLinkDown(BasePortTest):
    description = 'Test Port Link Down'

    def run(self):
        return self.ports_set('down')

class PortLLDPCheck(BasePortTest):
    description = 'Test Both ports are doing LLDP and recognize each other'
    def pre(self):
        err = self.ports_set('up')
        if err:
            return err
        for port in self.test_ports:
            port.lldpcli_update()
        return False
          
    def run(self):
        err = False
        for port in self.test_ports:            
            lldp_PortID = port.lldp_peer_port
            lldp_MgmtIP = port.lldp_peer_MgmtIP
            peer_MgmtIP = port.peer_switch.mgmtIP
            peer_PortID = port.peer_port.name

            if (lldp_PortID is None) or (lldp_MgmtIP is None):
                self.port_log_err(port, "lldpctl MgmtIP/PortID cannot be retrieved correctly")
                err = True
                continue

            if (lldp_MgmtIP != peer_MgmtIP) or (peer_PortID != lldp_PortID):
                self.port_log_err(port, "LLDP peer: %s %s" %(lldp_MgmtIP, lldp_PortID))
                self.port_log_err(port, "Provided link peer: %s %s" %(peer_MgmtIP, peer_PortID))
                self.port_log_err(port, "LLDP Peer and Provided link peer do not match")
                err = True
        return err

class PortEEPROMCheck(BasePortTest):
    description = 'Test if port EEPROMs can be read'

    def run(self):
        err = False
        for port in self.test_ports:
            if port.cage == 'RJ45':
                self.port_log(port, 'port is RJ45, skipping EEPROM check')
                continue
            elif port.identifier == None:
                self.port_log_err(port, 'Couldnt read EEPROM identifier')
                err = True
        return err

class PortAutoNegAndLinkUpCheck(BasePortTest):
    description = 'Set Autoneg on a port and see if link is UP'
    
    def run(self):
        err = self.ports_set('down')
        if err:
            self.log_err('Ports down failed on some ports')
            return err
        for port in self.test_ports:            
            port.autoneg = 'on'
        for port in self.test_ports:
            if port.autoneg != 'on':
                self.port_log_err(port, 'AN is not on')
                err = True
        if err:
            return err
        return self.ports_set('up')

    def post(self):
        self.log('Turning AN off on all ports as cleanup')
        for port in self.test_ports:
            port.autoneg = 'off'

class PortSpeed40And100GAndLinkUpCheck(BasePortTest):
    description = 'Set max speed on a QSFP+ or QSFP28 port'

    def pre(self):
        err = self.ports_set('down')
        if err:
            self.log_err('Not all ports went down')
            return err
        self.log('Just to be safe, setting Autoneg OFF on all ports')
        for port in self.test_ports:            
            port.autoneg = 'off'
        for port in self.test_ports:
            if port.autoneg != 'off':
                self.port_log_err(port, 'AN is not OFF')
                err = True
        if err:
            self.log_err('Autoneg is not OFF on all ports')
        return err
                                
    def run(self):
        err = False
        self.log('Setting max speed on all QSFP+ and QSFP28 ports')            
        for port in self.test_ports:
            cage = port.cage
            if cage.startswith('QSFP'):
                max_speed = port.max_speed
                self.port_log(port, 'Cage:%s identifier:%s setting max speed %s'
                              %(cage, port.identifier, max_speed))
                port.speed = max_speed
        self.log('Checking max speed is set on all ports')
        for port in self.test_ports:
            cage = port.cage 
            if cage.startswith('QSFP'):
                speed = port.speed
                max_speed = port.max_speed
                if speed != max_speed:
                    self.port_log_err(port,'speed %s not equal to max speed %s'
                                      %(speed, max_speed))
                    err = True
        if err:
            return err
        return self.ports_set('up')                    
    
class SwitchSwitchdRestart(BaseSwitchTest):
    description = 'Test switchd restart with links'    

    def pre(self):
        return self.ports_set('up')
            
    def run(self):
        err = self.switches_daemon_restart('switchd')
        if err:
            return True
        return self.ports_status('up')        

class SwitchPortwdRestart(BaseSwitchTest):
    description = 'Test portwd restart with links'    

    def pre(self):
        return self.ports_set('up')
            
    def run(self):        
        err = self.switches_daemon_restart('portwd')
        if err:
            return True
        return self.ports_status('up')        
            
# map of      test_name              : test_class
test_dict = { "PortLinkUp"           : PortLinkUp,
              "PortLinkDown"         : PortLinkDown,
              "SwitchSwitchdRestart" : SwitchSwitchdRestart,
              "SwitchPortwdRestart"  : SwitchPortwdRestart,
              "PortLinkFlap"         : PortLinkFlap,
              "PortLLDPCheck"        : PortLLDPCheck,
              "PortEEPROMCheck"      : PortEEPROMCheck,
              "PortAutoNegAndLinkUpCheck" : PortAutoNegAndLinkUpCheck,
              "PortSpeed40And100GAndLinkUpCheck" : PortSpeed40And100GAndLinkUpCheck,
            }    
                           
# Main helper functions                                                
def device_summary(config, port_dict, switch_dict):
    ''' Show info about links, ports and switches
    '''
    msg =  'Links:\n'
    msg += '======\n'   
    
    for link1,link2 in config.items('links'):        
        switch1,port1 = link1.split(',')
        switch2,port2 = link2.split(',')
        name1 = '%s,%s' %(switch1, port1)
        name2 = '%s,%s' %(switch2, port2)        
        msg += '%-25s <---> %-25s\n' %(name1, name2)

    msg += '\nPorts: (Cage, Identifier, Transreceiver Type, Vendor name, Vendor PN, Vendor SN, Length)\n'
    msg +=   '======\n'   
    for sw_name, switch in sorted(switch_dict.iteritems()):
        for pw_name, port in sorted(switch.ports.iteritems()):
            name = '%s,%s' %(sw_name, pw_name)
            msg += '%-25s (%s, %s, %s, %s, %s, %s, %s)\n' %(name, port.cage, port.identifier,
                                                        port.trans_type,
                                                        port.vendor_name,
                                                        port.vendor_pn, port.vendor_sn,
                                                        port.length)

    msg += '\nSwitches: (platform, chip, lsb-release)\n'
    msg +=   '========\n'
    for sw_name, switch in sorted(switch_dict.iteritems()):
        msg += '%-25s (%s, %s, %s)\n' %(sw_name, switch.platform, switch.chip, switch.release)
    return msg
            
def log(logf, msg):
    logf.write('%s %s\n' %(datetime.utcnow(), msg))

def get_switch_port_dict(config, logf=None):
    ''' From the config file collect all the links, create switch and port
        classes, map the right peers and return a dictionary of switches
        and ports.
    '''
    switch_dict = dict()
    port_dict = dict()    
    for link1,link2 in config.items('links'):        
        switch1,port1 = link1.split(',')
        switch2,port2 = link2.split(',')
        
        if not switch_dict.has_key(switch1):
            switch_dict[switch1] = Switch(switch1, logf)
        if not switch_dict.has_key(switch2):
            switch_dict[switch2] = Switch(switch2, logf)
        if not port_dict.has_key(link1):
            port_dict[link1] = Port(port1, switch_dict[switch1], logf)
        if not port_dict.has_key(link2):
            port_dict[link2] = Port(port2, switch_dict[switch2], logf)

        sw1 = switch_dict[switch1]
        sw2 = switch_dict[switch2]        
        p1 = port_dict[link1]
        p2 = port_dict[link2]
                
        p1.add_peer(switch_dict[switch2], p2)
        p2.add_peer(switch_dict[switch1], p1)
        sw1.add_port(p1)
        sw2.add_port(p2)

    return switch_dict, port_dict    
    
def get_tests_to_run(config):
    ''' From the config file, get all the tests to run
    '''
    global test_dict
    test_lower_dict = dict()
    p = dict()
    invalid_tests = []
    for name, dev in test_dict.iteritems():
        test_lower_dict[name.lower()] = dev
    for tst_name, dev in config.items('tests'):
        if test_lower_dict.has_key(tst_name):
            p[tst_name] = test_lower_dict[tst_name]
        else:
            invalid_tests.append(tst_name)
    return p, invalid_tests

def execute_test(test, switch_dict, port_dict, devices, logf):
    ''' Executes a test. Goes through the 3 stages: Pre, Run
       and Post and logs messages.
    '''
    tst = test(switch_dict, port_dict, devices, logf)
    name = tst.__class__.__name__
    log(logf, "%s: Starting Test" %name)
    if tst.pre():
        log(logf, "%s: Test failed at Pre stage" %name)
        return True
    if tst.run():
        log(logf, "%s: Test failed at Run stage" %name)
        return True
    if tst.post():
        log(logf, "%s: Test failed at Post stage" %name)
        return True
    log(logf, "%s: Test Passed" %name)    
    return False

def read_config(args, path):
    if not os.path.isfile(path):
        disp_msg('%s not found. No tests were run.' %path)
        sys.exit(-1)        
    config = ConfigParser.ConfigParser()
    config.read(path)
    return config
    
def main():
    log_path = os.path.join(os.getcwd(), 'testports.log')
    parser = argparse.ArgumentParser(description="Test Switch Ports")
    parser.add_argument('-p', '--path',
                        help = "Path to pick config file from. Default: %s" %os.path.join(os.getcwd(), 'testports.cfg'),
                        default=os.path.join(os.getcwd(), 'testports.cfg'))
    parser.add_argument('-l', '--links', action="store_true",
                        help = "Show Switch and Port Links")
    parser.add_argument('-t', '--tests', action="store_true",
                        help = "Show Tests to specify in config file")    
    parser.add_argument('-r', '--runtests', action="store_true",
                        help = "Run Tests")
    args = parser.parse_args()

    if args.tests:
        global test_dict        
        for tst_name, tst in test_dict.iteritems():
            print '%15s: %s' %(tst_name, tst.description)
        sys.exit(0)
                            
    if args.links:
        config = read_config(args, args.path)
        switch_dict, port_dict = get_switch_port_dict(config)
        print device_summary(config, port_dict, switch_dict)
        sys.exit(0)
    
    if args.runtests:        
        failed_tst = []
        passed_tst = []
        logf = open(log_path, 'w')
        print 'Logs are at: %s\n' %log_path

        # Read the config file, and create the required Switches and Ports classes.
        config = read_config(args, args.path)
        switch_dict, port_dict = get_switch_port_dict(config, logf)

        # Print what we discovered
        msg = device_summary(config, port_dict, switch_dict)
        print msg
        log(logf, msg)

        # Read the config file and create the required Tests classes.
        tests, invalids = get_tests_to_run(config)
        if len(invalids) > 0:
            print "Invalid Tests: %s" %invalids
            log(logf, "Following tests are invalid: %s\n" %(invalids))

        # Switch License Test
        no_lic_list = []
        for sw_name, sw in switch_dict.iteritems():
            if sw.no_license:
                no_lic_list.append(sw_name)
        if len(no_lic_list):
            print 'Please install license on %s followed by switchd restart' %(no_lic_list)
            log(logf, 'Please install license on %s followed by switchd restart' %(no_lic_list))
            sys.exit(1)

        # Run the tests
        print 'Running the following tests:'
        for name, test in tests.iteritems():
            print "%s" %name
            if execute_test(test, switch_dict, port_dict, 'ALL', logf):
                failed_tst.append(name)
            else:
                passed_tst.append(name)

        # Show the results.
        print '\nResults:'
        print  '======='   
        print 'Test Passed:%d %s' %(len(passed_tst), passed_tst)
        log(logf, 'Tests Passed:%d %s' %(len(passed_tst), passed_tst))
        print 'Test Failed:%d %s' %(len(failed_tst), failed_tst) 
        log(logf, 'Tests Failed:%d %s' %(len(failed_tst), failed_tst))
        logf.close()
        sys.exit(0)
        
    parser.print_help()
        
if __name__ == '__main__':
    main()

